//********************************************************************
//*** ResourcePool/ResourcePoolEventObserver.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: ResourcePoolEventObserver.t.cc,v 1.5 2009-03-02 14:16:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePoolEventObserver.hh"
#include "ResourcePoolStreamLogger.hh"
#include "../t/main.hh"
#include "../mm/gc_ptr.hh"

typedef fatalmind::ResourcePoolEventObserver::event_t event_t;

void TEST_EventNames() {
	fatalmind::ResourcePoolEventObserver eo;
	assertEqual(eo.getEvent("INCREMENT_POOL"), eo.Event.INCREMENT_POOL);
	assertEqual(eo.getEventName(eo.Event.INCREMENT_POOL), "INCREMENT_POOL");
	assertEqual(eo.getEvent("DOWNSIZE_POOL"), eo.Event.DOWNSIZE_POOL);
	assertEqual(eo.getEvent("FAIL_RESOURCE"), eo.Event.FAIL_RESOURCE);
	assertEqual(eo.getEvent("GET_RESOURCE_FAIL"), eo.Event.GET_RESOURCE_FAIL);
	assertEqual(eo.getEvent("EXECUTE_NFE"), eo.Event.EXECUTE_NFE);
	assertEqual(eo.getEvent("EXECUTE_FAIL"), eo.Event.EXECUTE_FAIL);
	assertEqual(eo.getEvent("FAIL"), eo.Event.FAIL);
	assertEqual(eo.getEvent("ALL"), eo.Event.ALL);
}

void TEST_EventNameMask() {
	fatalmind::ResourcePoolEventObserver eo;
	int byConstant = eo.Event.INCREMENT_POOL | eo.Event.DOWNSIZE_POOL;
	int byNames    = eo.getEventMask("INCREMENT_POOL | DOWNSIZE_POOL");
	assertEqual(byConstant, byNames);


	byConstant = eo.Event.EXECUTE_NFE | eo.Event.EXECUTE_NFE;
	byNames    = eo.getEventMask("EXECUTE_NFE | EXECUTE_NFE");
	assertEqual(byConstant, byNames);
}

void TEST_RegisterEvent() {
	fatalmind::ResourcePoolEventObserver eo;
	event_t event = eo.registerEvent("TESTEVENT1");
	assertNotEqual(event, eo.Event.NONE);
	event_t event2 = eo.registerEvent("TESTEVENT1");
	assertNotEqual(event2, eo.Event.NONE);
	assertEqual(event, event2);

	event2 = eo.registerEvent("TESTEVENT2");
	assertEqual(event*2, event2); // the next bit

	event2 = eo.registerEvent("TESTEVENT1");
	assertEqual(event, event2);
	std::string en1 = eo.getEventName(event);
	assertEqual(en1, "TESTEVENT1");

	for (int i = 0; i <= std::numeric_limits<event_t>::digits; ++i) {
		std::ostringstream os;
		os << "OVERFLOWTEST " << i;
		event = eo.registerEvent(os.str());
	}
	assertEqual(event, eo.Event.NONE); // must reach the limit until invalid
}

void TEST_PERF_EventReader_1000000() {
	fatalmind::ResourcePoolStreamLogger log(std::cout);
	log.setLogEvents(log.Event.NONE);
	for (int i = 0; i <= 1000000; ++i) {
		gc_ptr<fatalmind::EventReader> r(log.getReader(log.Event.INCREMENT_POOL, "N/A"));
		(*r) << "help " << i << " events logged";
	}
}

void TEST_PERF_EventReader_stderr_10000() {
	fatalmind::ResourcePoolStreamLogger log(std::cerr);
	for (int i = 0; i <= 10000; ++i) {
		gc_ptr<fatalmind::EventReader> r(log.getReader(log.Event.INCREMENT_POOL, "N/A"));
		(*r) << "help " << i << " events logged";
	}
}

void TEST_PERF_event_1000000() {
	fatalmind::ResourcePoolStreamLogger log(std::cout);
	log.setLogEvents(log.Event.NONE);
	for (int i = 0; i <= 1000000; ++i) {
		std::ostringstream os;
		os << "help " << i << " events logged";
		log.event(log.Event.INCREMENT_POOL, "N/A", os.str());
	}
}

