//********************************************************************
//*** ResourcePool/SQL/BindOut.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: BindOut.cc,v 1.20 2009-03-02 14:16:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <string>
#include <sstream>
#include "BindOut.hh"

namespace fatalmind {
namespace SQL {

template<class VBC, template<class> class CT>
BindOut<VBC, CT>::~BindOut() {
}

template<class VBC, template<class> class CT>
template<typename T>
void BindOut<VBC, CT>::bindout(const unsigned int pos, T& val, bool& isnull) {
	gc_ptr<VBC> ptr(new CT<T>(val, isnull));
	_outparam.resize(std::max<int>(pos + 1, _outparam.size()));
	_outparam[pos] = ptr;
	setIterators();
}

template<class VBC, template<class> class CT>
template<typename T>
void BindOut<VBC, CT>::bindout(const unsigned int pos, T& val, BindOutNotNullConstraint) {
	bindout(pos, val, BindOutInterface::notnullconstant);
	setIterators();
}

template<class VBC, template<class> class CT>
VBC& BindOut<VBC, CT>::getOutParam(const unsigned int pos) {
	if (pos < _outparam.size() && _outparam[pos].get()) {
		return *(_outparam[pos]);
	} else {
		std::ostringstream os;
		os << "bindout(" << pos << ", ...) has not been called";
		throw SQLException(os.str(), "07001", "UKN", false);
	}
}

} // namespace SQL
} // namespece fatalmind
