//********************************************************************
//*** ResourcePool/SQL/BindOutInterface.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: BindOutInterface.hh,v 1.6 2009-03-02 14:16:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_BINDOUTINTERFACE_HH
#define RESOURCEPOOL_SQL_BINDOUTINTERFACE_HH

namespace fatalmind {
namespace SQL {

/**
 * This class is for documentation purposes only.
 * It's not possible to use this polimorphycally since
 * the methods are not virtual (and can not be virtual since
 * it is a member template).
 */
class BindOutInterface {
	public:
		static bool notnullconstant;
		/**
		 * This inner class is used as parameter type to declar a column
		 * can not be NULL.
		 */
		class BindOutNotNullConstraint {
		};
		static BindOutNotNullConstraint NOTNULL;

		/**
		 * Specify where the output data of an select statement
		 * are to be stored.
		 *
		 * You need to specify an place where the columns for an
		 * select statement are to be stored. After each fetch
		 * those variables will be propagated with the actual values
		 * received from the select statement. 
		 * You can also specify a bool which will be used as indicator
		 * if the fatched column is NULL. If you omit this third, optional
		 * parameter an Exception will be thrown in case a fetched column
		 * is NULL.
		 *
		 * \param pos The column number to be bound. Numbering starts with
		 *        the leftmost column at 0.
		 * \param val The variable which should contain the value of this
		 *        column after the fetch. 
		 *        Each time a row is retrieved the according column value
		 *        value will be available in that variable.
		 *        The content of the variable will not be changed in case
 		 *        the column is NULL or the execution failed because of
                 *        an exception.
		 *        Exclusive access is
		 *        expected, so there are no synchronization effords taken
		 *        by the select commands. The variable must be availalbe
		 *        throughout the whole time of execution.
		 * \param isnull The optional third argument specifies a bool 
		 *        variable which will be set to true if the fetched 
		 *        column is NULL or to false otherwise.
		 *        You must supply this indicator variable if you might
		 *        retrieve NULL for this column.
		 *        Not supplying an indicator varaible will cause an\n
		 *        <tt>Indicator variable required but not supplied.</tt>
		 *        \n exception.
		 */
		template<typename T>
		void bindout(const unsigned int pos, T& val, bool& isnull);

		template<typename T>
		void bindout(const unsigned int pos, T& val, BindOutNotNullConstraint = NOTNULL);
};

} // namespace SQL
} // namespace fatalmind
#endif
