//********************************************************************
//*** ResourcePool/SQL/RTSQLBatch.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLBatch.hh,v 1.3 2009-03-02 14:16:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RTSQLBATCH_HH
#define RESOURCEPOOL_SQL_RTSQLBATCH_HH

#include "mm/gc_ptr.hh"
#include "ResourcePool/Command.hh"
#include "ResourcePool/SQL/SQLBatchCommand.hh"
#include "ResourcePool/SQL/Resource.hh"

namespace fatalmind {
namespace SQL {

template<class TM> class RTSQLBatch_Impl;

template<class TM>
class RTSQLBatch
: public SQLBatchCommand<Command<ResourceType<TM> >  >
{
	protected:
		typedef SQLBatchCommand<Command<ResourceType<TM> > > super;
	public:
		typedef typename super::innercommand_t innercommand_t;

		typedef typename super::const_iterator const_iterator;

		RTSQLBatch(const ResourcePool<ResourceType<TM> >& p, const bool cont = false);
		RTSQLBatch(const RTSQLBatch<TM>& rho);
		virtual ~RTSQLBatch();
		
		virtual void execute(typename ResourceType<TM>::resource_t & hand);

		virtual int getError();
		virtual void add(innercommand_t const& cmd);
		virtual const_iterator getExecPosition() const;

		virtual bool doesContinue() const;
		virtual const_iterator end() const;
		virtual const_iterator begin() const;

		virtual bool empty() const;

		virtual void outputoperator(std::ostream&) const;

	protected:
		virtual Clone* DoClone() const;

	private:
		typedef RTSQLBatch_Impl<TM> rimpl_t;
		typedef typename SQLFactory<TM>::databaseType_t databaseType_t;
		databaseType_t dbType;
		gc_ptr<rimpl_t> _realSQLBatch;	
};


} // namespace SQL
} // namespace fatalmind
#endif
