//********************************************************************
//*** ResourcePool/SQL/RTSQLTransaction_Impl.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLTransaction_Impl.hh,v 1.6 2009-03-02 14:16:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RTSQLTRANSACTION_IMPL_HH
#define RESOURCEPOOL_SQL_RTSQLTRANSACTION_IMPL_HH

#include "config.h"
#include "util/Clone.hh"

#include "ResourcePool/SQL/Factory.hh"

#ifdef HAVE_MYSQL_TRANSACTIONS
#include "ResourcePool/mysql/ResourcePool.hh"
#endif

#ifdef HAVE_ORACLE
#include "ResourcePool/oracle/ResourcePool.hh"
#include "ResourcePool/oracle/Resource.hh"
#include "ResourcePool/oracle/oracleType.hh"
#endif


#include <sstream>

namespace fatalmind {
namespace SQL {

template<class TM>
class RTSQLTransaction_Impl: public Clone {
	public:
		RTSQLTransaction_Impl() {
		};
		virtual ~RTSQLTransaction_Impl() {
		}

		virtual void execute(SQLResource<TM>& hand) = 0;

		template<class CS>
		static typename CS::Impl* newRealCommand(
			  typename SQLFactory<TM>::databaseType_t a_type
			, typename CS::RTC& a_that
			, const typename CS::RTC::innercommand_t& a_cmd
		) {
			switch(a_type) {
#ifdef HAVE_MYSQL
			case SQLFactory<TM>::MYSQLTX :
#ifdef HAVE_MYSQL_TRANSACTIONS
				return new typename CS::template Specialized<fatalmind::ResourceType::mysql>::Impl(a_that, a_cmd);
#else
				throw Exception("This version of ResourcePool was not build with mysql transaction support");
#endif
				break;
			case SQLFactory<TM>::MYSQL :
				// TODO, nop emulate
				throw Exception("This version of ResourcePool was not build with mysql transaction support");
				break;
#endif
#ifdef HAVE_ORACLE
			case SQLFactory<TM>::OCI:
				return new typename CS::template Specialized<fatalmind::ResourceType::oracle>::Impl(a_that, a_cmd);
				break;
#endif
			default:
				std::ostringstream oss;
				oss << "Assertaion failure (type=" << a_type << ")";
				throw Exception(oss.str());
				break;
		}
	}
};

template<class TM, template<class TM2>class B, class RT, class CW, class RTC>
class RTSQLTransaction_SpecializedImplCloneHelper
{
	public:
	static Clone* DoClone(const Clone * const that) {
		throw CloneException(that);	
	}
};

// fwd decl
template<class TM, template<class TM2>class B, class RT, class CW, class RTC>
class RTSQLTransaction_SpecializedImpl ;

template<class TM, class RT, class CW, class RTC>
class RTSQLTransaction_SpecializedImplCloneHelper<TM, RTSQLTransaction_Impl, RT, CW, RTC>
{
	public:
	typedef  RTSQLTransaction_SpecializedImpl<TM, RTSQLTransaction_Impl, RT, CW, RTC> target;
	static Clone* DoClone(const Clone *const that) {
		const target *x = dynamic_cast<const target*>(that);	
		return new target(*x);
	}
};


template<class TM, template<class TM2>class B, class RT, class CW, class RTC>
class RTSQLTransaction_SpecializedImpl 
: public B<TM>
{
	public:
		RTSQLTransaction_SpecializedImpl(RTC& rtc, const typename RTC::innercommand_t& cmd) 
		: sqlTransaction_(rtc, cmd)
		{
		}

		virtual void execute(SQLResource<TM>& hand) {
			sqlTransaction_.execute(
				dynamic_cast<
					SQLResourceSpecialized<TM, typename RT::resource_t>&
				>(hand).getRealResource()
			);
		}

	protected:
		virtual Clone* DoClone() const {
			return RTSQLTransaction_SpecializedImplCloneHelper<TM, B, RT, CW, RTC>::DoClone(this);
		}

		CW sqlTransaction_;
};


} // namespace SQL
} // namespace fatalmind

#endif
