//********************************************************************
//*** ResourcePool/oracle/OCIHandle.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: OCIHandle.hh,v 1.22 2009-03-02 14:16:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_OCIHANDLE_HH
#define RESOURCEPOOL_ORACLE_OCIHANDLE_HH

#ifndef RESOURCEPOOL_ORACLE_OCIENVHANDLE_HH
#include "OCIEnvHandle.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCEFWD_HH
#include "ResourceFwd.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLESTATEMENT_HH
#include "oracleStatement.hh"
#endif

#ifndef OCI_ORACLE
#include <oci.h>
#endif

#include <string>
#include <memory>

namespace fatalmind {
namespace oracle {

template<typename HTYPE, int HCONST>
class OCIHandle {
	public:
		OCIHandle(const OCIEnv* env);
		OCIHandle(const OCIEnvHandle& env);
		~OCIHandle();
		inline HTYPE* get() const {
			return  _h;
		};
	private:
		HTYPE* _h;
};

typedef oracle::OCIHandle<OCIError,   OCI_HTYPE_ERROR>   OCIErrorHandle;
typedef oracle::OCIHandle<OCIServer,  OCI_HTYPE_SERVER>  OCIServerHandle;
typedef oracle::OCIHandle<OCISvcCtx,  OCI_HTYPE_SVCCTX>  OCISvcCtxHandle;
typedef oracle::OCIHandle<OCISession, OCI_HTYPE_SESSION> OCISessionHandle;

class OCIStatementHandle {
	public:
		OCIStatementHandle(const oracleStatement&);
		~OCIStatementHandle() {
		}
		bool prepare(oracleHandles&);
		OCIStmt* get() const;
		unsigned int getEfficienty() const;

		std::string getSQL() const;
	private:
		void force_prepare(oracleHandles&);

		std::string _SQL;
		typedef OCIHandle<OCIStmt, OCI_HTYPE_STMT> Stmt_t;
		std::auto_ptr<Stmt_t> _sth;

		unsigned int _prepare;
		unsigned int _force_prepare;

		/**
		 * Indicates if the cursor can be cached.
		 * As of current suspection, only SELECT
		 * INSERT or DELETE statements should be cached,
		 * CREATE/DROP may have funny effects.
		 */
		bool isCacheable(oracleHandles&) const;

	friend
		std::ostream& operator<<(std::ostream& s, const OCIStatementHandle&);
};

std::ostream& operator<<(std::ostream& s, const OCIStatementHandle&);

} // namespace oracle
} // namespace fatalmind
#endif
