//********************************************************************
//*** ResourcePool/oracle/OracleExceptionFactory.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: OracleExceptionFactory.hh,v 1.14 2009-03-02 14:16:46 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

// TODO - move into fatalmind::oracle namespace

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#define RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTION_HH
#include "OracleException.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_OCIHANDLE_HH
#include "OCIHandle.hh"
#endif

#include <sstream>

namespace fatalmind {

class OracleExceptionFactory {
	public:
		// TODO: i think some of them never have a meaningfull
		// sqlstm, those should not take the sqlstm argument
		// and default it to "N/A"
		static void throwit(const std::string& op
			, const std::string& fac
			, const int errorcode
			, const std::string& msg
			, const std::string& sqlstm
		);
		static void throwit(
			  const std::string& op
			, const oracle::OCIEnvHandle& env
			, sword ret
			, const std::string& sqlstm
		);
		static void throwit(
			  const std::string& op
			, const oracle::OCIErrorHandle& err
			, sword ret
			, const std::string& sqlstm
		);
		static void throwit(
			  const std::string& op
			, const oracleHandles&
			, sword ret
			, const std::string& sqlstm
		);
		
	// TODO
	//private:
		static void throwit(const std::string& oper
			, const dvoid* errhp
			, sword ret
			, ub4 type
			, const std::string& sqlstm
		);

		/**
		 * The non-static access is required if the exception is to be thrown delayed.
		 * Upon construction all required data is obtained from the database and stored
		 * internally.
		 * at a later time the non-static throw() method can be used to actually
		 * throw the exception.
		 */
		OracleExceptionFactory(
			  const std::string& oper
			, const dvoid* errhp
			, sword ret
			, ub4 type
			, const std::string& sqlstm
		);

		OracleExceptionFactory(
			  const std::string& op
			, const OCIError* errhp
			, sword ret
			, const std::string& sqlstm
		);

		// returns the string provided by OCI to describe the errro
		std::string getErrorString() const;
		
		// returns the code (typically ORA-xxxxx) of the error
		std::string getErrorCode() const;

	
		void throwit() const;

	private:
		void setFromOCI(const std::string& oper, const dvoid* errhp, ub4 type, sword ret);
		std::string state;
		std::string msg;
		std::string sqlstm;
		int		    errorcode;
};

} // namespace fatalmind

#endif
