//********************************************************************
//*** Thread/TaskRunner.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: TaskRunner.t.cc,v 1.13 2009-03-02 14:16:46 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "t/main.hh"
#include "TaskRunner.hh"

using namespace fatalmind;

class myTestTask: public Task {
	public:
		myTestTask()
		: exec(0)
		{
		}

		virtual void run() {
			++exec;
		}
		int exec;
};

void testit(int n) {
	TaskRunner runner;
	gc_ptr<myTestTask, MultiThreadedModel<> > t(new myTestTask());	
	TaskRunner::ID id; 

	assertEqual(t->exec, 0);
	runner.run(t);
	id = runner.run(t);
	id.wait();
	assertEqual(t->exec, 2);
	for (int i = 2; i < n; ++i) {
		runner.run(t);
	}
	runner.stop();
	assertEqual(t->exec, n);
	try {
		runner.run(t);
		assertNotReach();
	} catch (...) {
	}
}

void testit_detached(int n) {
	TaskRunner runner;
	gc_ptr<myTestTask, MultiThreadedModel<> > t(new myTestTask());	
	TaskRunner::ID id;

	assertEqual(t->exec, 0);
	runner.runDetached(t);
	id = runner.run(t);
	id.wait();
	assertEqual(t->exec, 2);
	for (int i = 2; i < n; ++i) {
		runner.runDetached(t);
	}
	runner.stop();
	assertEqual(t->exec, n);
	try {
		runner.runDetached(t);
		assertNotReach();
	} catch (...) {
	}
}

void TEST_TaskRunner_one() {
	testit(5);
}

void TEST_TaskRunner_one_detached() {
	testit_detached(5);
}

void TEST_PERF_TaskRunner_five() {
	for (int i = 0; i < 1000; ++i) {
		testit(5);
	}
}

void TEST_PERF_TaskRunner_fifity() {
	for (int i = 0; i < 100; ++i) {
		testit(50);
	}
}

void TEST_PERF_TaskRunner_fivehundred() {
	for (int i = 0; i < 10; ++i) {
		testit(500);
	}
}
void TEST_PERF_TaskRunner_five_detached() {
	for (int i = 0; i < 1000; ++i) {
		testit_detached(5);
	}
}

void TEST_PERF_TaskRunner_fifity_detached() {
	for (int i = 0; i < 100; ++i) {
		testit_detached(50);
	}
}

void TEST_PERF_TaskRunner_fivehundred_detached() {
	for (int i = 0; i < 10; ++i) {
		testit_detached(500);
	}
}

void TEST_TaskRunner_two() {
	TaskRunner runner;
	TaskRunner runner2;
	std::auto_ptr<myTestTask> t(new myTestTask());	
	std::auto_ptr<myTestTask> t2(new myTestTask());	

	runner.runDetached(t);
	runner.runDetached(t2);
	
	runner.stop();
	runner2.stop();
	
}
